"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var NodeModel =
/*#__PURE__*/
function () {
  function NodeModel(props) {
    var nodes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck2.default)(this, NodeModel);
    this.props = props;
    this.flatNodes = nodes;
  }

  (0, _createClass2.default)(NodeModel, [{
    key: "setProps",
    value: function setProps(props) {
      this.props = props;
    }
  }, {
    key: "clone",
    value: function clone() {
      var _this = this;

      var clonedNodes = {}; // Re-construct nodes one level deep to avoid shallow copy of mutable characteristics

      Object.keys(this.flatNodes).forEach(function (value) {
        var node = _this.flatNodes[value];
        clonedNodes[value] = (0, _objectSpread2.default)({}, node);
      });
      return new NodeModel(this.props, clonedNodes);
    }
  }, {
    key: "getNode",
    value: function getNode(value) {
      return this.flatNodes[value];
    }
  }, {
    key: "flattenNodes",
    value: function flattenNodes(nodes) {
      var _this2 = this;

      var parent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      if (!Array.isArray(nodes) || nodes.length === 0) {
        return;
      }

      var _this$props = this.props,
          disabled = _this$props.disabled,
          noCascade = _this$props.noCascade; // Flatten the `node` property for internal lookups

      nodes.forEach(function (node, index) {
        var isParent = _this2.nodeHasChildren(node);

        _this2.flatNodes[node.value] = {
          label: node.label,
          value: node.value,
          children: node.children,
          parent: parent,
          isParent: isParent,
          isLeaf: !isParent,
          showCheckbox: node.showCheckbox !== undefined ? node.showCheckbox : true,
          disabled: _this2.getDisabledState(node, parent, disabled, noCascade),
          treeDepth: depth,
          index: index
        };

        _this2.flattenNodes(node.children, node, depth + 1);
      });
    }
  }, {
    key: "nodeHasChildren",
    value: function nodeHasChildren(node) {
      return Array.isArray(node.children) && node.children.length > 0;
    }
  }, {
    key: "getDisabledState",
    value: function getDisabledState(node, parent, disabledProp, noCascade) {
      if (disabledProp) {
        return true;
      }

      if (!noCascade && parent.disabled) {
        return true;
      }

      return Boolean(node.disabled);
    }
  }, {
    key: "deserializeLists",
    value: function deserializeLists(lists) {
      var _this3 = this;

      var listKeys = ['checked', 'expanded']; // Reset values to false

      Object.keys(this.flatNodes).forEach(function (value) {
        listKeys.forEach(function (listKey) {
          _this3.flatNodes[value][listKey] = false;
        });
      }); // Deserialize values and set their nodes to true

      listKeys.forEach(function (listKey) {
        lists[listKey].forEach(function (value) {
          if (_this3.flatNodes[value] !== undefined) {
            _this3.flatNodes[value][listKey] = true;
          }
        });
      });
    }
  }, {
    key: "serializeList",
    value: function serializeList(key) {
      var _this4 = this;

      var list = [];
      Object.keys(this.flatNodes).forEach(function (value) {
        if (_this4.flatNodes[value][key]) {
          list.push(value);
        }
      });
      return list;
    }
  }, {
    key: "expandAllNodes",
    value: function expandAllNodes(expand) {
      var _this5 = this;

      Object.keys(this.flatNodes).forEach(function (value) {
        if (_this5.flatNodes[value].isParent) {
          _this5.flatNodes[value].expanded = expand;
        }
      });
      return this;
    }
  }, {
    key: "toggleChecked",
    value: function toggleChecked(node, isChecked, noCascade) {
      var _this6 = this;

      var flatNode = this.flatNodes[node.value];

      if (flatNode.isLeaf || noCascade) {
        if (node.disabled) {
          return this;
        } // Set the check status of a leaf node or an uncoupled parent


        this.toggleNode(node.value, 'checked', isChecked);
      } else {
        // Percolate check status down to all children
        flatNode.children.forEach(function (child) {
          _this6.toggleChecked(child, isChecked, noCascade);
        });
      }

      return this;
    }
  }, {
    key: "toggleNode",
    value: function toggleNode(nodeValue, key, toggleValue) {
      this.flatNodes[nodeValue][key] = toggleValue;
      return this;
    }
  }]);
  return NodeModel;
}();

var _default = NodeModel;
exports.default = _default;